﻿-- CREATE LANGUAGE 'plpgsql';

-- DROP FUNCTION f_cc_veranstaltunsgueberschriften(INTEGER);

-- Creates the list of absolute "Ueberschriften" paths to which a "Veranstaltung"
-- has being assigned
CREATE OR REPLACE FUNCTION f_cc_veranstaltungsueberschriften(veranstid_in INTEGER) RETURNS TABLE (r_leafueid INTEGER, r_path TEXT) AS
$BODY$
BEGIN
	RETURN QUERY
	WITH RECURSIVE r_hierarchie_cte(semester, parent_ueid, path, leaf_ueid) AS (
		SELECT veranstaltung.semester,
			r_hierarchie.uebergeord,
			ueberschrift.txt::TEXT,
			r_hierarchie.untergeord
		FROM veranstaltung,
			r_vvzzuord
			INNER JOIN ueberschrift ON r_vvzzuord.ueid=ueberschrift.ueid
			INNER JOIN r_hierarchie ON ueberschrift.ueid=r_hierarchie.untergeord
		WHERE veranstaltung.veranstid=veranstid_in
			AND r_vvzzuord.veranstid=veranstid_in
			AND r_hierarchie.semester=veranstaltung.semester
			AND r_hierarchie.tabelle='ueberschrift'
	 UNION ALL
		SELECT child.semester,
			parent.uebergeord,
			(CASE WHEN parent.uebergeord IS NULL THEN child.path ELSE (ueberschrift.txt::TEXT || ' &amp;#x27ff; ' || child.path) END),
			child.leaf_ueid
		FROM r_hierarchie_cte AS child
			INNER JOIN r_hierarchie AS parent ON parent.untergeord = child.parent_ueid
			LEFT JOIN ueberschrift ON parent.untergeord = ueberschrift.ueid
		WHERE parent.semester=child.semester
			AND parent.tabelle='ueberschrift'
	)
	SELECT leaf_ueid, path
	FROM r_hierarchie_cte
	WHERE parent_ueid IS NULL;
END;
$BODY$
LANGUAGE plpgsql;


-- Test:
-- SELECT * FROM
--	f_cc_veranstaltungsueberschriften(87629); -- In our test-db, it returns 4 results